# coding=utf-8
""""""
from __future__ import absolute_import, division, print_function, \
    unicode_literals

from wtforms.fields import BooleanField, StringField

from abilian.i18n import _l
from abilian.services.security.models import Role
from abilian.web.forms import Form, fields, widgets
from abilian.web.forms.filters import strip
from abilian.web.forms.validators import required


class GroupAdminForm(Form):
    name = StringField(_l('Name'), filters=(strip,), validators=[required()])
    description = StringField(_l('Description'), filters=(strip,))

    public = BooleanField(
        _l('Public'),
        widget=widgets.BooleanWidget(on_off_mode=True),
    )

    roles = fields.Select2MultipleField(
        _l('Roles'),
        choices=lambda: [(r.name, r.label) for r in Role.assignable_roles()],
    )



# coding=utf-8
"""
"""
from __future__ import absolute_import, print_function, unicode_literals

from flask_script import Manager

from abilian.core.commands import setup_abilian_commands
from demo.app import create_app

manager = Manager(create_app)


setup_abilian_commands(manager)


def main():
    manager.run()


if __name__ == "__main__":
    main()



# coding=utf-8
""""""
from __future__ import absolute_import, division, print_function, \
    unicode_literals

from flask_login import current_user, login_user
from pytest import fixture

from abilian.app import Application as BaseApplication
from abilian.core.models.subjects import User
from abilian.services import get_service

from .models import UserPreference
from .panel import PreferencePanel
from .service import PreferenceService


class VisiblePanel(PreferencePanel):
    id = label = 'visible'

    def is_accessible(self):
        return True

    def get(self):
        return 'Visible'


class AdminPanel(PreferencePanel):
    id = label = 'admin'

    def is_accessible(self):
        security = get_service('security')
        return security.has_role(current_user, "admin")

    def get(self):
        return 'Admin'


class Application(BaseApplication):

    def init_extensions(self):
        super(Application, self).init_extensions()
        prefs = self.services['preferences']
        prefs.app_state.panels = []
        prefs.register_panel(VisiblePanel(), self)
        prefs.register_panel(AdminPanel(), self)


@fixture
def app(config):
    return Application(config=config)


def test_preferences(app, session):
    user = User(email="test@example.com")
    assert UserPreference.query.all() == []

    preference_service = PreferenceService()

    preferences = preference_service.get_preferences(user)
    assert preferences == {}

    preference_service.set_preferences(user, digest='daily')
    session.flush()

    preferences = preference_service.get_preferences(user)
    assert preferences == {'digest': 'daily'}

    preference_service.clear_preferences(user)
    session.flush()

    preferences = preference_service.get_preferences(user)
    assert preferences == {}
    assert UserPreference.query.all() == []


def test_preferences_with_various_types(app, session):
    user = User(email="test@example.com")
    preference_service = PreferenceService()

    preference_service.set_preferences(user, some_int=1)
    session.flush()
    preferences = preference_service.get_preferences(user)
    assert preferences == {'some_int': 1}

    preference_service.set_preferences(user, some_bool=True)
    session.flush()
    preferences = preference_service.get_preferences(user)
    assert preferences == {'some_int': 1, 'some_bool': True}


def test_visible_panels(app, db):
    user = User(email="test@example.com")
    security = app.services['security']

    with app.test_request_context():
        security.start()
        login_user(user)

        for cp in app.template_context_processors['preferences']:
            ctx = cp()
            if 'menu' in ctx:
                break

        expected = ['preferences.visible']
        assert [p['endpoint'] for p in ctx['menu']] == expected

        security.grant_role(user, 'admin')
        ctx = cp()
        expected = ['preferences.visible', 'preferences.admin']
        assert [p['endpoint'] for p in ctx['menu']] == expected



# coding=utf-8
"""
Configuration and injectable fixtures for Pytest.

Supposed to replace the too-complex current UnitTest-based testing
framework.

DI and functions over complex inheritance hierarchies FTW!
"""
from __future__ import absolute_import, division, print_function, \
    unicode_literals

from pytest import fixture
from sqlalchemy.exc import DatabaseError

from abilian.app import create_app
from abilian.services import get_service


class TestConfig:
    TESTING = True
    SERVER_NAME = 'localhost'
    CELERY_ALWAYS_EAGER = True  # run tasks locally, no async
    CELERY_EAGER_PROPAGATES_EXCEPTIONS = True
    MAIL_SENDER = 'tester@example.com'
    SITE_NAME = "Abilian Test"
    CSRF_ENABLED = True
    WTF_CSRF_ENABLED = True


@fixture
def config():
    return TestConfig


@fixture
def app(config):
    # We currently return a fresh app for each test.
    # Using session-scoped app doesn't currently work.
    # Note: the impact on speed is minimal.
    return create_app(config=config)


@fixture
def app_context(app):
    with app.app_context() as ctx:
        yield ctx


@fixture
def test_request_context(app):
    with app.test_request_context() as ctx:
        yield ctx


@fixture
def db(app, app_context):
    """Return a fresh db for each test."""
    from abilian.core.extensions import db

    stop_all_services(app)
    ensure_services_started(['repository', 'session_repository'])

    cleanup_db(db)
    db.create_all()
    yield db

    db.session.remove()
    cleanup_db(db)
    stop_all_services(app)


@fixture
def session(db):
    return db.session


@fixture
def client(app, db):
    """Return a Web client, used for testing, bound to a DB session."""
    return app.test_client()


#
# Cleanup utilities
#
def cleanup_db(db):
    """Drop all the tables, in a way that doesn't raise integrity errors."""

    # Need to run this sequence twice for some reason
    for i in range(0, 2):
        delete_tables(db)
    # One more time, just in case ?
    db.drop_all()


def delete_tables(db):
    for table in reversed(db.metadata.sorted_tables):
        try:
            db.session.execute(table.delete())
        except DatabaseError:
            pass


def stop_all_services(app):
    for service in app.services.values():
        if service.running:
            service.stop()


def ensure_services_started(services):
    for service_name in services:
        service = get_service(service_name)
        if not service.running:
            service.start()



# coding=utf-8
""""""
from __future__ import absolute_import, division, print_function, \
    unicode_literals

from flask_login import AnonymousUserMixin, LoginManager


class AnonymousUser(AnonymousUserMixin):

    def has_role(self, role):
        from abilian.services import get_service

        security = get_service('security')
        return security.has_role(self, role)

    @property
    def groups(self):
        return set()


login_manager = LoginManager()
login_manager.anonymous_user = AnonymousUser
